@@ -89,6 +89,9 @@ static char *option_argument = NULL;
    DB_META_DATA_FILE_BASE.  */
 static char *db_file_name = NULL;
 
+/* Use a "smart store proxy".  */
+static int smart_storage = 1;
+
 /* Use the dummy store for debugging purposes.  */
 static int debugging = 0;
 
@@ -127,14 +130,12 @@ static int use_dbus = 0;
 #endif
 
 
-#ifdef HAVE_GPERF
 static char *file_based_store_class_name = "gdbm_block_store";
 static char *chopper_class_name = "fixed_size_chopper";
 static char *indexer_class_name = "tree_indexer";
 static char *indexer_ascii = "64"; /* 100 keys per block */
 static char *block_indexer_class_name = "hash_block_indexer";
 static char *block_indexer_ascii = "SHA1";
-#endif
 
 static struct argp_option options[] =
   {
@@ -142,6 +143,9 @@ static struct argp_option options[] =
     { "debug",   'd', 0, 0,
       "Produce debugging output and use a dummy block store (i.e. a block "
       "store that does nothing but print messages)" },
+    { "no-smart-store", 'N', 0, 0,
+      "Disable smart storage, which writes data blocks only when they don't "
+      "already exist" },
     { "show-stats", 's', 0, 0,
       "Show statistics about the blocks that have been written (in archival "
       "mode)" },
@@ -177,7 +181,7 @@ static struct argp_option options[] =
       "Use PRIVKEY-FILE as the OpenPGP key to be used during TLS "
       "authentication" },
 #endif
-#ifdef HAVE_GPERF
+
     { "store",   'S', "CLASS", 0,
       "Use CLASS as the underlying file-based block store" },
     { "chopper", 'C', "CHOPPER", 0,
@@ -190,7 +194,6 @@ static struct argp_option options[] =
       "Use BI-CLASS as the block-indexer class.  This implies `-I'." },
     { "block-indexer", 'I', "BI", 0,
       "Deserialize BI as an instance of BI-CLASS and use it." },
-#endif
 
 #ifdef HAVE_DBUS
     { "dbus", 'D', 0, 0,
@@ -221,16 +224,6 @@ do_archive (chop_stream_t *stream, chop_block_store_t *data_store,
   chop_block_indexer_t *block_indexer;
   chop_index_handle_t *handle;
 
-#ifndef HAVE_GPERF
-  block_indexer = chop_class_alloca_instance (&chop_hash_block_indexer_class);
-  err = chop_hash_block_indexer_open (CHOP_HASH_SHA1,
-				      block_indexer);
-  if (err)
-    {
-      chop_error (err, "while opening hash block indexer");
-      return err;
-    }
-#else
   {
     size_t bytes_read = 0;
     const chop_class_t *block_indexer_class;
@@ -267,7 +260,6 @@ do_archive (chop_stream_t *stream, chop_block_store_t *data_store,
       fprintf (stderr, "%s: warning: %zu: trailing garbage in block-indexer\n",
 	       program_name, bytes_read);
   }
-#endif
 
   handle = chop_block_indexer_alloca_index_handle (block_indexer);
   err = chop_indexer_index_blocks (indexer, chopper, block_indexer,
@@ -472,7 +464,6 @@ process_command (const char *argument,
 	    chop_log_attach (chop_filter_log (zip_filter), 2, 0);
 	}
 
-#ifdef HAVE_GPERF
       indexer_class = chop_class_lookup (indexer_class_name);
       if (!indexer_class)
 	{
@@ -487,9 +478,6 @@ process_command (const char *argument,
 		   program_name, indexer_class_name);
 	  exit (1);
 	}
-#else
-      indexer_class = &chop_tree_indexer_class;
-#endif
 
       indexer = chop_class_alloca_instance (indexer_class);
       err = chop_object_deserialize ((chop_object_t *)indexer,
@@ -506,7 +494,6 @@ process_command (const char *argument,
       if (verbose)
 	log_indexer (indexer);
 
-#ifdef HAVE_GPERF
       chopper_class =
 	(chop_chopper_class_t *)chop_class_lookup (chopper_class_name);
       if (!chopper_class)
@@ -522,9 +509,6 @@ process_command (const char *argument,
 		   program_name, chopper_class_name);
 	  exit (1);
 	}
-#else
-      chopper_class = &chop_fixed_size_chopper_class;
-#endif
 
       chopper = chop_class_alloca_instance ((chop_class_t *)chopper_class);
       err = chop_chopper_generic_open (chopper_class, stream,
@@ -585,6 +569,7 @@ process_command (const char *argument,
 	{
 	  chop_log_t *fetcher_log = chop_hash_block_fetcher_log (fetcher);
 
+	  fetcher_log = fetcher_log ?: chop_chk_block_fetcher_log (fetcher);
 	  if (fetcher_log)
 	    chop_log_attach (fetcher_log, 2, 0);
 
@@ -691,6 +676,9 @@ parse_opt (int key, char *arg, struct argp_state *state)
     case 's':
       show_stats = 1;
       break;
+    case 'N':
+      smart_storage = 0;
+      break;
     case 'f':
       db_file_name = arg;
       break;
@@ -748,7 +736,6 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
 #endif
 
-#ifdef HAVE_GPERF
     case 'S':
       file_based_store_class_name = arg;
       break;
@@ -767,7 +754,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
     case 'I':
       block_indexer_ascii = arg;
       break;
-#endif
+
 #ifdef HAVE_DBUS
     case 'D':
       use_dbus = 1;
@@ -852,7 +839,7 @@ main (int argc, char *argv[])
 	{
 	  /* Use two GDBM stores.  */
 	  const chop_file_based_store_class_t *db_store_class;
-#ifdef HAVE_GPERF
+
 	  db_store_class = (chop_file_based_store_class_t *)
 	    chop_class_lookup (file_based_store_class_name);
 	  if (!db_store_class)
@@ -869,9 +856,6 @@ main (int argc, char *argv[])
 		       argv[0], file_based_store_class_name);
 	      exit (1);
 	    }
-#else
-	  db_store_class = &chop_gdbm_block_store_class;
-#endif
 
 	  store = (chop_block_store_t *)
 	    chop_class_alloca_instance ((chop_class_t *)db_store_class);
@@ -968,6 +952,40 @@ main (int argc, char *argv[])
 	}
     }
 
+  if (smart_storage && archive_queried)
+    {
+      chop_block_store_t *data_proxy, *metadata_proxy;
+
+      data_proxy =
+	chop_class_alloca_instance (&chop_smart_block_store_class);
+
+      err = chop_smart_block_store_open (store,
+					 CHOP_PROXY_EVENTUALLY_DESTROY,
+					 data_proxy);
+      if (!err)
+	{
+	  if (store != metastore)
+	    {
+	      metadata_proxy =
+		chop_class_alloca_instance (&chop_smart_block_store_class);
+
+	      err = chop_smart_block_store_open (metastore,
+						 CHOP_PROXY_EVENTUALLY_DESTROY,
+						 metadata_proxy);
+	    }
+	  else
+	    metadata_proxy = data_proxy;
+	}
+
+      if (err)
+	{
+	  chop_error (err, "while creating smart store proxy");
+	  exit (EXIT_FAILURE);
+	}
+      else
+	store = data_proxy, metastore = metadata_proxy;
+    }
+
   if (show_stats)
     {
       /* Create ``statistic stores'' proxying both block stores.  */
